@@ -20,15 +20,17 @@ class common_debian_package_command(Command):
         self.debian_version = None
         self.no_backwards_compatibility = None
         self.guess_conflicts_provides_replaces = None
-        if sys.version_info.major==2:
+        if sys.version_info[0]==2:
             self.with_python2 = 'True'
             self.with_python3 = 'False'
         else:
-            assert sys.version_info.major==3
+            assert sys.version_info[0]==3
             self.with_python2 = 'False'
             self.with_python3 = 'True'
         self.no_python2_scripts = 'False'
         self.no_python3_scripts = 'False'
+        self.force_x_python3_version = False
+        self.allow_virtualenv_install_location = False
 
         # deprecated options
         self.default_distribution = None
@@ -65,6 +67,21 @@ class common_debian_package_command(Command):
         self.with_python3 = str_to_bool(self.with_python3)
         self.no_python2_scripts = str_to_bool(self.no_python2_scripts)
         self.no_python3_scripts = str_to_bool(self.no_python3_scripts)
+        if self.maintainer is not None:
+            # Get the locale specifying the encoding in sys.argv
+            import locale, codecs
+            fs_enc = codecs.lookup(locale.getpreferredencoding()).name
+            if hasattr(os,'fsencode'): # this exists only in Python 3
+                m = os.fsencode(self.maintainer) # convert to original raw bytes
+
+                # Now, convert these raw bytes into unicode.
+                m = m.decode(fs_enc) # Set your locale if you get errors here
+
+                self.maintainer = m
+            else:
+                # Python 2
+                if hasattr(self.maintainer,'decode'):
+                    self.maintainer = self.maintainer.decode(fs_enc)
 
     def get_debinfo(self):
         ###############################################
@@ -91,6 +108,9 @@ class common_debian_package_command(Command):
                      '--default-maintainer option. '
                      'Switch to the --maintainer option.')
             guess_maintainer = self.default_maintainer
+        if hasattr(guess_maintainer,'decode'):
+            # python 2 : convert (back to) unicode
+            guess_maintainer = guess_maintainer.decode('utf-8')
 
         #    B. find config files (if any)
         cfg_files = []
@@ -163,6 +183,19 @@ class common_debian_package_command(Command):
                 raise ValueError("Illegal character (%r) detected in version. "
                                  "This will break the debian tools."%bad_char)
 
+        description = self.distribution.get_description()
+        if hasattr(description,'decode'):
+            # python 2 : convert (back to) unicode
+            description = description.decode('utf-8')
+        description = description[:60]
+
+        long_description = self.distribution.get_long_description()
+        if hasattr(long_description,'decode'):
+            # python 2 : convert (back to) unicode
+            long_description = long_description.decode('utf-8')
+        long_description = long_description
+
+
         debinfo = DebianInfo(
             cfg_files=cfg_files,
             module_name = module_name,
@@ -170,8 +203,8 @@ class common_debian_package_command(Command):
             guess_maintainer=guess_maintainer,
             upstream_version = upstream_version,
             has_ext_modules = self.distribution.has_ext_modules(),
-            description = self.distribution.get_description()[:60],
-            long_description = self.distribution.get_long_description(),
+            description = description,
+            long_description = long_description,
             patch_file = self.patch_file,
             patch_level = self.patch_level,
             debian_version = self.debian_version,
@@ -184,5 +217,7 @@ class common_debian_package_command(Command):
             with_python3 = self.with_python3,
             no_python2_scripts = self.no_python2_scripts,
             no_python3_scripts = self.no_python3_scripts,
+            force_x_python3_version=self.force_x_python3_version,
+            allow_virtualenv_install_location=self.allow_virtualenv_install_location,
         )
         return debinfo
