@@ -2,6 +2,7 @@
 # This module contains most of the code of stdeb.
 #
 import re, sys, os, shutil, select
+import codecs
 try:
     # Python 2.x
     import ConfigParser
@@ -44,11 +45,11 @@ def check_call(*popenargs, **kwargs):
         return
     raise CalledProcessError(retcode)
 
-if sys.version_info.major==2:
+if sys.version_info[0]==2:
     help_str_py2='If True, build package for python 2. (Default=True).'
     help_str_py3='If True, build package for python 3. (Default=False).'
 else:
-    assert sys.version_info.major==3
+    assert sys.version_info[0]==3
     help_str_py2='If True, build package for python 2. (Default=False).'
     help_str_py3='If True, build package for python 3. (Default=True).'
 
@@ -100,6 +101,11 @@ stdeb_cmdline_opts = [
      'If True, do not install scripts for python 2. (Default=False).'),
     ('no-python3-scripts=',None,
      'If True, do not install scripts for python 3. (Default=False).'),
+    ('force-x-python3-version',None,
+     'Override default minimum python3:any dependency with value from '
+     'x-python3-version'),
+    ('allow-virtualenv-install-location',None,
+     'Allow installing into /some/random/virtualenv-path'),
     ]
 
 # old entries from stdeb.cfg:
@@ -140,12 +146,18 @@ stdeb_cfg_options = [
     ('depends=',None,'debian/control Depends:'),
     ('depends3=',None,'debian/control Depends:'),
     ('suggests=',None,'debian/control Suggests:'),
+    ('suggests3=',None,'debian/control Suggests:'),
     ('recommends=',None,'debian/control Recommends:'),
+    ('recommends3=',None,'debian/control Recommends:'),
     ('xs-python-version=',None,'debian/control XS-Python-Version:'),
+    ('x-python3-version=',None,'debian/control X-Python3-Version:'),
     ('dpkg-shlibdeps-params=',None,'parameters passed to dpkg-shlibdeps'),
     ('conflicts=',None,'debian/control Conflicts:'),
+    ('conflicts3=',None,'debian/control Conflicts:'),
     ('provides=',None,'debian/control Provides:'),
+    ('provides3=',None,'debian/control Provides3:'),
     ('replaces=',None,'debian/control Replaces:'),
+    ('replaces3=',None,'debian/control Replaces3:'),
     ('mime-desktop-files=',None,'MIME desktop files'),
     ('mime-file=',None,'MIME file'),
     ('shared-mime-file=',None,'shared MIME file'),
@@ -159,6 +171,8 @@ stdeb_cmd_bool_opts = [
     'patch-posix',
     'ignore-install-requires',
     'no-backwards-compatibility',
+    'force-x-python3-version',
+    'allow-virtualenv-install-location',
     ]
 
 class NotGiven: pass
@@ -229,13 +243,23 @@ def get_cmd_stdout(args):
         raise RuntimeError('returncode %d', returncode)
     return cmd.stdout.read()
 
+def normstr(s):
+    try:
+        # Python 3.x
+        result = str(s,'utf-8')
+    except TypeError:
+        # Python 2.x
+        result = s
+    return result
+
 def get_date_822():
     """return output of 822-date command"""
     cmd = '/bin/date'
     if not os.path.exists(cmd):
         raise ValueError('%s command does not exist.'%cmd)
     args = [cmd,'-R']
-    result = get_cmd_stdout(args).strip().decode('ascii')
+    result = get_cmd_stdout(args).strip()
+    result = normstr(result)
     return result
 
 def get_version_str(pkg):
@@ -679,6 +703,8 @@ class DebianInfo:
                  with_python3 = None,
                  no_python2_scripts = None,
                  no_python3_scripts = None,
+                 force_x_python3_version=False,
+                 allow_virtualenv_install_location=False,
                  ):
         if cfg_files is NotGiven: raise ValueError("cfg_files must be supplied")
         if module_name is NotGiven: raise ValueError(
@@ -706,7 +732,9 @@ class DebianInfo:
             check_cfg_files(cfg_files,module_name)
 
         cfg = ConfigParser.SafeConfigParser(cfg_defaults)
-        cfg.read(cfg_files)
+        for cfg_file in cfg_files:
+            with codecs.open( cfg_file, mode='r', encoding='utf-8') as fd:
+                cfg.readfp(fd)
 
         if sdist_dsc_command is not None:
             # Allow distutils commands to override config files (this lets
@@ -850,7 +878,9 @@ class DebianInfo:
         self.build_depends = ', '.join(build_deps)
 
         suggests = ', '.join( parse_vals(cfg,module_name,'Suggests') )
+        suggests3 = ', '.join( parse_vals(cfg,module_name,'Suggests3') )
         recommends = ', '.join( parse_vals(cfg,module_name,'Recommends') )
+        recommends3 = ', '.join( parse_vals(cfg,module_name,'Recommends3') )
 
         self.source_stanza_extras = ''
 
@@ -887,6 +917,13 @@ class DebianInfo:
             self.source_stanza_extras += ('X-Python-Version: '+
                                           ', '.join(xs_python_version)+'\n')
 
+        x_python3_version = parse_vals(cfg,module_name,'X-Python3-Version')
+        x_python3_version = [v.strip('"') for v in x_python3_version]
+
+        if len(x_python3_version)!=0:
+            self.source_stanza_extras += ('X-Python3-Version: '+
+                                          ', '.join(x_python3_version)+'\n')
+
         dpkg_shlibdeps_params = parse_val(
             cfg,module_name,'dpkg-shlibdeps-params')
         if dpkg_shlibdeps_params:
@@ -899,8 +936,11 @@ class DebianInfo:
         self.dh_binary_indep_lines = '\tdh binary-indep'
 
         conflicts = parse_vals(cfg,module_name,'Conflicts')
+        conflicts3 = parse_vals(cfg,module_name,'Conflicts3')
         provides = parse_vals(cfg,module_name,'Provides')
+        provides3 = parse_vals(cfg,module_name,'Provides3')
         replaces = parse_vals(cfg,module_name,'Replaces')
+        replaces3 = parse_vals(cfg,module_name,'Replaces3')
 
         if guess_conflicts_provides_replaces:
             # Find list of binaries which we will conflict/provide/replace.
@@ -923,8 +963,11 @@ class DebianInfo:
             for orig_binary in cpr_binaries:
                 for version_info in apt_cache_info('show',orig_binary):
                     provides.extend( version_info['Provides'])
+                    provides3.extend( version_info['Provides'])
                     conflicts.extend(version_info['Conflicts'])
+                    conflicts3.extend(version_info['Conflicts'])
                     replaces.extend( version_info['Replaces'])
+                    replaces3.extend( version_info['Replaces'])
 
             if self.package in cpr_binaries:
                 cpr_binaries.remove(self.package) # don't include ourself
@@ -932,13 +975,19 @@ class DebianInfo:
             cpr_binaries = list(cpr_binaries) # convert to list
 
             conflicts.extend( cpr_binaries )
+            conflicts3.extend( cpr_binaries )
             provides.extend( cpr_binaries )
+            provides3.extend( cpr_binaries )
             replaces.extend( cpr_binaries )
+            replaces3.extend( cpr_binaries )
 
             # round-trip through set to get unique entries
             conflicts = list(set(conflicts))
+            conflicts3 = list(set(conflicts3))
             provides = list(set(provides))
+            provides3 = list(set(provides3))
             replaces = list(set(replaces))
+            replaces3 = list(set(replaces3))
 
         self.package_stanza_extras = ''
         self.package_stanza_extras3 = ''
@@ -946,20 +995,36 @@ class DebianInfo:
         if len(conflicts):
             self.package_stanza_extras += ('Conflicts: '+
                                               ', '.join( conflicts )+'\n')
+        if len(conflicts3):
+            self.package_stanza_extras3 += ('Conflicts: '+
+                                              ', '.join( conflicts3 )+'\n')
 
         if len(provides):
             self.package_stanza_extras += ('Provides: '+
                                              ', '.join( provides  )+'\n')
 
+        if len(provides3):
+            self.package_stanza_extras3 += ('Provides: '+
+                                             ', '.join( provides3  )+'\n')
+
         if len(replaces):
             self.package_stanza_extras += ('Replaces: ' +
                                               ', '.join( replaces  )+'\n')
+        if len(replaces3):
+            self.package_stanza_extras3 += ('Replaces: ' +
+                                              ', '.join( replaces3 )+'\n')
         if len(recommends):
             self.package_stanza_extras += ('Recommends: '+recommends+'\n')
 
+        if len(recommends3):
+            self.package_stanza_extras3 += ('Recommends: '+recommends3+'\n')
+
         if len(suggests):
             self.package_stanza_extras += ('Suggests: '+suggests+'\n')
 
+        if len(suggests3):
+            self.package_stanza_extras3 += ('Suggests: '+suggests3+'\n')
+
         self.dirlist = ""
 
         if not (with_python2 or with_python3):
@@ -991,12 +1056,14 @@ class DebianInfo:
         self.scripts_cleanup = '\n'.join(['        '+s for s in no_script_lines])
 
         if sys.prefix != '/usr':
-            # virtualenv will set distutils --prefix=/path/to/virtualenv, but
-            # we want to install into /usr.
-            workaround_virtualenv_distutils = True
-            self.install_prefix = '--prefix=/usr'
+            if not allow_virtualenv_install_location:
+                # virtualenv will set distutils
+                # --prefix=/path/to/virtualenv, but unless explicitly
+                # requested, we want to install into /usr.
+                self.install_prefix = '--prefix=/usr'
+            else:
+                self.install_prefix = '--prefix=%s' % sys.prefix
         else:
-            workaround_virtualenv_distutils = False
             self.install_prefix = ''
 
         rules_override_clean_target_pythons = []
@@ -1029,6 +1096,23 @@ class DebianInfo:
         self.override_dh_auto_clean = RULES_OVERRIDE_CLEAN_TARGET%self.__dict__
         self.override_dh_auto_build = RULES_OVERRIDE_BUILD_TARGET%self.__dict__
         self.override_dh_auto_install = RULES_OVERRIDE_INSTALL_TARGET%self.__dict__
+
+        if force_x_python3_version and with_python3 and x_python3_version and \
+                x_python3_version[0]:
+            # override dh_python3 target to modify the dependencies
+            # to ensure that the passed minimum X-Python3-Version is usedby
+            version = x_python3_version[0]
+            if not version.endswith('~'):
+                version += '~'
+            self.override_dh_python3 = RULES_OVERRIDE_PYTHON3%{
+                'scripts': (
+                    '        sed -i ' +
+                    '"s/\([ =]python3:any (\)>= [^)]*\()\)/\\1%s\\2/g" ' +
+                    'debian/%s.substvars') % (version, self.package3)
+            }
+        else:
+            self.override_dh_python3 = ''
+
         sequencer_options = ['--with '+','.join(sequencer_with)]
         sequencer_options.append('--buildsystem=python_distutils')
         self.sequencer_options = ' '.join(sequencer_options)
@@ -1183,14 +1267,10 @@ def build_dsc(debinfo,
         os.mkdir(debian_dir)
 
     #    A. debian/changelog
-    fd = open( os.path.join(debian_dir,'changelog'), mode='w')
-    fd.write("""\
-%(source)s (%(full_version)s) %(distname)s; urgency=low
-
-  * source package automatically created by stdeb %(stdeb_version)s
-
- -- %(maintainer)s  %(date822)s\n"""%debinfo.__dict__)
-    fd.close()
+    changelog = CHANGELOG_FILE%debinfo.__dict__
+    with codecs.open( os.path.join(debian_dir,'changelog'),
+                 mode='w', encoding='utf-8') as fd:
+        fd.write(changelog)
 
     #    B. debian/control
     if debinfo.uploaders:
@@ -1198,9 +1278,9 @@ def build_dsc(debinfo,
     else:
         debinfo.uploaders = ''
     control = CONTROL_FILE%debinfo.__dict__
-    fd = open( os.path.join(debian_dir,'control'), mode='w')
-    fd.write(control)
-    fd.close()
+    with codecs.open( os.path.join(debian_dir,'control'),
+                      mode='w', encoding='utf-8') as fd:
+        fd.write(control)
 
     #    C. debian/rules
     debinfo.percent_symbol = '%'
@@ -1208,9 +1288,9 @@ def build_dsc(debinfo,
 
     rules = rules.replace('        ','\t')
     rules_fname = os.path.join(debian_dir,'rules')
-    fd = open( rules_fname, mode='w')
-    fd.write(rules)
-    fd.close()
+    with codecs.open( rules_fname,
+                      mode='w', encoding='utf-8') as fd:
+        fd.write(rules)
     os.chmod(rules_fname,0o755)
 
     #    D. debian/compat
@@ -1349,6 +1429,13 @@ def build_dsc(debinfo,
         dpkg_source('-x',dsc_name,
                     cwd=dist_dir)
 
+CHANGELOG_FILE = """\
+%(source)s (%(full_version)s) %(distname)s; urgency=low
+
+  * source package automatically created by stdeb %(stdeb_version)s
+
+ -- %(maintainer)s  %(date822)s\n"""
+
 CONTROL_FILE = """\
 Source: %(source)s
 Maintainer: %(maintainer)s
@@ -1394,6 +1481,11 @@ RULES_MAIN = """\
 
 %(override_dh_auto_install)s
 
+override_dh_python2:
+        dh_python2 --no-guessing-versions
+
+%(override_dh_python3)s
+
 %(binary_target_lines)s
 """
 
@@ -1422,6 +1514,12 @@ override_dh_auto_install:
 %(scripts_cleanup)s
 """
 
+RULES_OVERRIDE_PYTHON3 = """
+override_dh_python3:
+        dh_python3
+%(scripts)s
+"""
+
 RULES_BINARY_TARGET = """
 binary: binary-arch binary-indep
 """
